SystemOrganization addCategory: #'COM-Core'!
SystemOrganization addCategory: #'COM-Interfaces'!
SystemOrganization addCategory: #'COM-Types'!
SystemOrganization addCategory: #'COM-Types-Tests'!

!Integer methodsFor: '*COM-Support' stamp: 'tbn 7/21/2003 21:35'!
hexString
	^ String with: ('0123456789ABCDEF' at: self//16+1)
			with:  ('0123456789ABCDEF' at: self\\16+1)! !

TestCase subclass: #COMGUIDTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'COM-Types-Tests'!

!COMGUIDTest methodsFor: 'testing' stamp: 'tbn 8/4/2004 23:12'!
testInstanceCreationFromProgID
	| guid |
	guid := self defaultTestClass fromProgID: 'WScript.Shell'.
	self assert: guid asString = '{72C24DD5-D70A-438B-8A42-98424B88AFB8}'.
	
	guid := self defaultTestClass fromProgID: 'Excel.Application'.
	self assert: guid asString = '{00024500-0000-0000-C000-000000000046}'.
	
	guid := self defaultTestClass fromProgID: 'Word.Application'.
	self assert: guid asString = '{000209FF-0000-0000-C000-000000000046}'.! !

!COMGUIDTest methodsFor: 'testing' stamp: 'tbn 8/2/2004 23:35'!
testInstanceCreation
	| guid |
	guid := self defaultTestClass new.
	self assert: guid size = 16! !

!COMGUIDTest methodsFor: 'testing' stamp: 'tbn 8/2/2004 23:35'!
testInstanceCreationFromBytes
	| guid |
	guid := self defaultTestClass 
				fromBytes: #(89 45 160 215 101 71 219 71 187 65 163 200 99 222 62 166).
	self assert: guid asString = '{D7A02D59-4765-47DB-BB41-A3C863DE3EA6}'! !

!COMGUIDTest methodsFor: 'testing' stamp: 'tbn 8/2/2004 23:35'!
testConversionToString
	| guid |
	guid := self defaultTestClass 
				fromBytes: #(233 128 172 104 244 238 8 74 180 49 211 208 110 58 63 4).
	self assert: guid asString = '{68AC80E9-EEF4-4A08-B431-D3D06E3A3F04}'! !

!COMGUIDTest methodsFor: 'attributes' stamp: 'tbn 8/2/2004 23:34'!
defaultTestClass
	^COMGUID! !

!COMGUIDTest methodsFor: 'testing' stamp: 'tbn 8/2/2004 23:34'!
testGenerate
	| guid |
	guid := self defaultTestClass generate.
	self assert: (guid isKindOf: self defaultTestClass)! !

Object subclass: #COMLibrary
	instanceVariableNames: ''
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'COM-Core'!

!COMLibrary methodsFor: 'api-calls-common' stamp: 'tbn 8/2/2004 23:26'!
initializeCOMEx: pReserved initValue: dwCoInit

	<apicall: long 'CoInitializeEx' (void* long) module: 'ole32.dll'>
	^self externalCallFailed! !

!COMLibrary class methodsFor: 'private instance creation' stamp: 'tbn 8/2/2004 23:30'!
createDefaultInstance
	"Returns a new initialized instance of the receiver"
	
	^(self new)
		initialize;
		yourself! !

!COMLibrary methodsFor: 'api-calls-guid handling' stamp: 'tbn 8/2/2004 23:42'!
coCreateGUID: byteArray  
	<apicall: long 'CoCreateGuid' (char*) module: 'ole32.dll'>
	^self externalCallFailed! !

!COMLibrary class methodsFor: 'accessing' stamp: 'tbn 8/2/2004 23:29'!
default
	"Returns the default instance of the receiver"
	
	Default isNil ifTrue: [Default := self createDefaultInstance].
	^Default ! !

!COMLibrary methodsFor: 'api-calls-guid handling' stamp: 'tbn 8/2/2004 23:51'!
clsidFromProgID: progID into: byteArray
	<apicall: long 'CLSIDFromProgID' (char* char*) module: 'ole32.dll'>
	^self externalCallFailed! !

!COMLibrary methodsFor: 'api-calls-common' stamp: 'tbn 7/21/2003 20:39'!
initializeCOM
	self 
		initializeCOMEx: nil 
		initValue: 2 "COINIT_APARTMENTTHREADED"
! !

!COMLibrary methodsFor: 'initialize-release' stamp: 'tbn 7/21/2003 20:15'!
initialize
	self initializeCOM! !

Object variableWordSubclass: #COMGUID
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'COM-Types'!

!COMGUID commentStamp: 'tbn 8/2/2004 23:32' prior: 0!
This class represents a COM Global Unique Identifier.!

!COMGUID class methodsFor: 'private accessing' stamp: 'tbn 8/2/2004 23:46'!
createBytes
	^ByteArray new: 16! !

!COMGUID class methodsFor: 'instance creation' stamp: 'tbn 7/21/2003 22:21'!
fromBytes: aCollection
	|instance|
	instance := self new.
	1 to: 16 do: [:index | instance at: index put: (aCollection at: index)].
	^instance! !

!COMGUID methodsFor: 'printing' stamp: 'tbn 7/21/2003 22:04'!
printHexStringAt: index on: aStream
	aStream nextPutAll: (self byteAt: index) hexString! !

!COMGUID methodsFor: 'accessing' stamp: 'tbn 7/21/2003 21:15'!
byteAt: index
	<primitive: 60>
	^self at: index! !

!COMGUID class methodsFor: 'accessing' stamp: 'tbn 7/21/2003 21:13'!
byteSize
	^16! !

!COMGUID methodsFor: 'printing' stamp: 'tbn 7/21/2003 22:35'!
asString
	| data stream |
	data := String new: 38.
	stream := WriteStream on: data.
	stream nextPut: ${.
	4 to: 1 by: - 1 do: [:index | self printHexStringAt: index on: stream].
	stream nextPut: $-.
	self
		printHexStringAt: 6 on: stream;
		printHexStringAt: 5 on: stream.
	stream nextPut: $-.
	self
		printHexStringAt: 8 on: stream;
		printHexStringAt: 7 on: stream.
	stream nextPut: $-.
	self
		printHexStringAt: 9 on: stream;
		printHexStringAt: 10 on: stream.
	stream nextPut: $-.
	11 to: 16 do: [:index | self printHexStringAt: index on: stream].
	stream nextPut: $}.
	^data! !

!COMGUID class methodsFor: 'instance creation' stamp: 'tbn 7/21/2003 21:21'!
new
	^self new: self byteSize! !

!COMGUID class methodsFor: 'instance creation' stamp: 'tbn 8/4/2004 23:07'!
fromProgID: aString 
	"Answers the class guid representing the given program id"

	| bytes |
	bytes := self createBytes.
	(self comLibrary clsidFromProgID: (COMWideString fromString: aString squeakToIso) into: bytes).
 
	^self fromBytes: bytes! !

!COMGUID class methodsFor: 'attributes' stamp: 'tbn 8/2/2004 23:37'!
comLibrary
	^COMLibrary default! !

!COMGUID methodsFor: 'printing' stamp: 'tbn 7/21/2003 22:19'!
printOn: aStream
	aStream nextPutAll: self asString
		
				
				
		 ! !

!COMGUID class methodsFor: 'generating' stamp: 'tbn 8/2/2004 23:46'!
generate
	"Generates a new unique instance of the receiver using an external call"

	| bytes |
	bytes := self createBytes.
	self comLibrary coCreateGUID: bytes.
	^self fromBytes: bytes! !

Object subclass: #COMIUnknown
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'COM-Interfaces'!

COMIUnknown subclass: #COMIDispatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'COM-Interfaces'!

COMIUnknown subclass: #COMIClassFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'COM-Interfaces'!

Object variableByteSubclass: #COMWideString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'COM-Types'!

!COMWideString class methodsFor: 'api-calls' stamp: 'tbn 8/4/2004 23:10'!
fromString: aString
	|wsSize result |
	wsSize := 
		self 
			multiByteToWideCharForPage: 0 	"CP_ACP -ANSI Code page"
			flags: 1						   	"MB_PRECOMPOSED - default translation option"
			mapString: aString
			count: aString size				"ansi string size"
			into: nil							"no buffer"	
			withSize: 0.						"function should return the required wide string size"
	 								
	result := ByteArray new: wsSize * 2.
	 self 
		multiByteToWideCharForPage: 0 	 
		flags: 1						   	 
		mapString: aString
		count: aString size				 
		into: result						
		withSize: wsSize * 2.
	^result   ! !

!COMWideString class methodsFor: 'api-calls' stamp: 'tbn 8/3/2004 00:25'!
multiByteToWideCharForPage: codePage flags: dwFlags mapString: multiByteString count: count into: buffer withSize: bufferSize 

	<apicall: long 'MultiByteToWideChar' (char char char* char char* char) module: 'kernel32.dll'>
	^self externalCallFailed! !
